// SPDX-License-Identifier: BSD-3-Clause
/* Copyright 2016, Intel Corporation */

/*
 * libpmempool_include.c -- include test for libpmempool
 *
 * this is only a compilation test - do not run this program
 */

#include <libpmempool.h>

int
main(int argc, char *argv[])
{
	return 0;
}
