/* SPDX-License-Identifier: BSD-3-Clause */
/* Copyright 2018-2020, Intel Corporation */

/*
 * auto_flush.h -- auto flush detection functionality
 */

#ifndef PMEM2_AUTO_FLUSH_H
#define PMEM2_AUTO_FLUSH_H 1

#ifdef __cplusplus
extern "C" {
#endif

int pmem2_auto_flush(void);

#ifdef __cplusplus
}
#endif

#endif
