// SPDX-License-Identifier: BSD-3-Clause
/* Copyright 2016, Intel Corporation */

/*
 * obj_base_include.c -- include test for libpmemobj
 */

#include <libpmemobj/base.h>

int
main(int argc, char *argv[])
{
	return 0;
}
