/* SPDX-License-Identifier: BSD-3-Clause */
/* Copyright 2018-2023, Intel Corporation */

/*
 * feature.h -- pmempool feature command header file
 */

int pmempool_feature_func(const char *appname, int argc, char *argv[]);
void pmempool_feature_help(const char *appname);
