/* SPDX-License-Identifier: BSD-3-Clause */
/* Copyright 2019-2020, Intel Corporation */

/*
 * pmem2.h -- internal definitions for libpmem2
 */
#ifndef PMEM2_H
#define PMEM2_H

#include "libpmem2.h"

#ifdef __cplusplus
extern "C" {
#endif

#define PMEM2_MAJOR_VERSION 0
#define PMEM2_MINOR_VERSION 0

#define PMEM2_LOG_PREFIX "libpmem2"
#define PMEM2_LOG_LEVEL_VAR "PMEM2_LOG_LEVEL"
#define PMEM2_LOG_FILE_VAR "PMEM2_LOG_FILE"

#ifdef __cplusplus
}
#endif

#endif
