# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2021, Intel Corporation

#
# src/libpmem2/riscv64/flags.inc -- flags for libpmem2/riscv64
#

vpath %.c $(TOP)/src/libpmem2/riscv64
vpath %.h $(TOP)/src/libpmem2/riscv64

CFLAGS += -Iriscv64
