# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2018, Intel Corporation

#
# src/libpmem2/aarch64/flags.inc -- flags for libpmem2/arm64
#

vpath %.c $(TOP)/src/libpmem2/aarch64
vpath %.h $(TOP)/src/libpmem2/aarch64

CFLAGS += -Iaarch64
