# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2014-2024, Intel Corporation

# Name of package
PACKAGE_NAME="pmdk"

# Name and email of package maintainer
PACKAGE_MAINTAINER="Piotr Balcer <piotr.balcer@intel.com>"

# Brief description of the package
PACKAGE_SUMMARY="Persistent Memory Development Kit"

# Full description of the package
PACKAGE_DESCRIPTION="The collection of libraries and utilities for Persistent Memory Programming"

# Website
PACKAGE_URL="https://github.com/pmem/pmdk"
