# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2022, Intel Corporation

#
# src/libpmem2/loongarch64/flags.inc -- flags for libpmem2/loongarch64
#

vpath %.c $(TOP)/src/libpmem2/loongarch64
vpath %.h $(TOP)/src/libpmem2/loongarch64

CFLAGS += -Iloongarch64
